""" SonnenAPI v2 module """
import asyncio
import functools

import datetime
from time import time
import aiohttp
import logging

import requests


def get_item(func):
    """ Decorator for getting json data from Sonnen API """
    @functools.wraps(func)
    def inner(*args):
        try:
            result = func(*args)
        except KeyError:
            print(f'{func} key not found')
            result = None
        return result
    return inner


class Sonnen:
    """Class for managing Sonnen API data"""
    # API Groups
    IC_STATUS = 'ic_status'

    # API Item keys
    CONSUMPTION_KEY = 'Consumption_W'
    PRODUCTION_KEY = 'Production_W'
    GRID_FEED_IN_WATT_KEY = 'GridFeedIn_W'
    USOC_KEY = 'USOC'
    RSOC_KEY = 'RSOC'
    BATTERY_CHARGE_OUTPUT_KEY = 'Apparent_output'
    REM_CON_WH_KEY = 'RemainingCapacity_Wh'
    PAC_KEY = 'Pac_total_W'
    SECONDS_SINCE_FULL_KEY = 'secondssincefullcharge'
    MODULES_INSTALLED_KEY = 'nrbatterymodules'
    CONSUMPTION_AVG_KEY = 'Consumption_Avg'
    FULL_CHARGE_CAPACITY_KEY = 'FullChargeCapacity'
    BATTERY_CYCLE_COUNT = 'cyclecount'
    BATTERY_FULL_CHARGE_CAPACITY = 'fullchargecapacity'
    BATTERY_MAX_CELL_TEMP = 'maximumcelltemperature'
    BATTERY_MAX_CELL_VOLTAGE = 'maximumcellvoltage'
    BATTERY_MAX_MODULE_CURRENT = 'maximummodulecurrent'
    BATTERY_MAX_MODULE_VOLTAGE = 'maximummoduledcvoltage'
    BATTERY_MAX_MODULE_TEMP = 'maximummoduletemperature'
    BATTERY_MIN_CELL_TEMP = 'minimumcelltemperature'
    BATTERY_MIN_CELL_VOLTAGE = 'minimumcellvoltage'
    BATTERY_MIN_MODULE_CURRENT = 'minimummodulecurrent'
    BATTERY_MIN_MODULE_VOLTAGE = 'minimummoduledcvoltage'
    BATTERY_MIN_MODULE_TEMP = 'minimummoduletemperature'
    BATTERY_RSOC = 'relativestateofcharge'
    BATTERY_REMAINING_CAPACITY = 'remainingcapacity'
    BATTERY_SYSTEM_CURRENT = 'systemcurrent'
    BATTERY_SYSTEM_VOLTAGE = 'systemdcvoltage'
    SYSTEM_STATUS = 'statecorecontrolmodule'
    # default timeout
    TIMEOUT = aiohttp.ClientTimeout(total=5)

    def __init__(self, auth_token: str,
                 ip_address: str,
                 logger: logging.Logger = None) -> None:
        self._last_updated = None
        self.ip_address = ip_address
        self.logger = logger
        self.auth_token = auth_token
        self.url = f'http://{ip_address}'
        self.header = {'Auth-Token': self.auth_token}

        # read api endpoints
        self.status_api_endpoint = f'{self.url}/api/v2/status'
        self.latest_details_api_endpoint = f'{self.url}/api/v2/latestdata'
        self.battery_api_endpoint = f'{self.url}/api/v2/battery'

        # api data
        self._latest_details_data = {}
        self._status_data = {}
        self._ic_status = {}
        self._battery_status = {}

    def _log_error(self, msg):
        if self.logger:
            self.logger.error(msg)
        else:
            print(msg)

    def _fetch_data(self, url: str) -> dict:
        """Fetches data from API endpoint"""
        try:
            response = requests.get(url, headers=self.header, timeout=self.TIMEOUT.total)
            if response.status_code == 200:
                data = response.json()
                return data
        except ConnectionError as error:
            self._log_error(f'Connection error to the battery system!')
            return {}

    def fetch_latest_details(self) -> bool:
        """Fetches latest details api"""
        try:
            self._latest_details_data = self._fetch_data(self.latest_details_api_endpoint)
            return True
        except Exception as error:
            self._log_error(f'Error while data parsing for {self.latest_details_api_endpoint} {error}')
        return False

    def fetch_status(self):
        """Fetches status api"""
        try:
            self._status_data = self._fetch_data(self.status_api_endpoint)
            return True
        except Exception as error:
            self._log_error(f'Error while data parsing for {self.status_api_endpoint}')
        return False

    def fetch_battery_status(self):
        """Fetches battery api endpoint"""
        try:
            self._battery_status = self._fetch_data(self.battery_api_endpoint)
            return True
        except Exception as error:
            self._log_error(f'Error while data parsing for {self.battery_api_endpoint}{error}')

    async def _async_fetch_data(self, url: str) -> dict:
        """Fetches data from the API endpoint with asyncio"""
        try:
            async with aiohttp.ClientSession(headers=self.header) as session:
                resp = await session.get(url)
                data = resp.json()
                return await data
        except aiohttp.ClientError as error:
            self._log_error(f'Battery: {self.ip_address} is offline!')
        except asyncio.TimeoutError as error:
            self._log_error(f'Timeout error while accessing: {url}')
        except Exception as error:
            self._log_error(f'Error while data parsing {error}')
            return {}

    async def async_fetch_latest_details(self) -> bool:
        """Fetches latest details api as coroutine"""
        try:
            self._latest_details_data = await self._async_fetch_data(
                self.latest_details_api_endpoint
            )
            self._ic_status = self._latest_details_data[self.IC_STATUS]
            return True
        except Exception as error:
            self._log_error(f'Error occurred while data parsing latest details:{error}')
            return False

    async def async_fetch_status(self) -> bool:
        """Fetches status api as coroutine"""
        try:
            self._status_data = await self._async_fetch_data(
                self.latest_details_api_endpoint
            )
            return True
        except Exception as error:
            self._log_error(f'Error occurred while data parsing status:{error}')
            return False

    async def async_fetch_battery_status(self) -> bool:
        """Fetches battery details api as coroutine"""
        try:
            self._battery_status = await self._async_fetch_data(
                self.battery_api_endpoint
            )
            return True
        except Exception as error:
            self._log_error(f'Error occurred while data parsing battery status:{error}')
        return False

    async def async_update(self) -> bool:
        """Updates data from apis of the sonnenBatterie as coroutine"""
        success = await self.async_fetch_latest_details()
        success = success and await self.async_fetch_status()
        success = success and await self.async_fetch_battery_status()
        self.last_updated = datetime.datetime.now()
        return success

    def update(self) -> bool:
        """Updates data from apis of the sonnenBatterie"""
        success = self.fetch_latest_details()
        success = success and self.fetch_status()
        success = success and self.fetch_battery_status()
        self.last_updated = datetime.datetime.now()
        return success
    
    @property
    def last_updated(self) -> time:
        return self._last_updated

    @last_updated.setter
    def last_updated(self, last_updated: time):
        self._last_updated = last_updated

    @get_item
    def consumption(self) -> int:
        """Consumption of the household
            Returns:
                house consumption in Watt
        """
        return self._latest_details_data[self.CONSUMPTION_KEY]

    @get_item
    def consumption_average(self) -> int:
        """Average consumption in watt
           Returns:
               average consumption in watt
        """

        return self._status_data[self.CONSUMPTION_AVG_KEY]

    @get_item
    def production(self) -> int:
        """Power production of the household
            Returns:
                house production in Watt
        """
        return self._latest_details_data[self.PRODUCTION_KEY]

    def seconds_to_empty(self) -> int:
        """Time until battery discharged
            Returns:
                Time in seconds
        """
        seconds = int((self.remaining_capacity_wh() / self.discharging()) * 3600) if self.discharging() else 0

        return seconds

    def fully_discharged_at(self) -> datetime:
        """Future time of battery fully discharged
            Returns:
                Future time
        """
        if self.discharging():
            return (datetime.datetime.now() + datetime.timedelta(seconds=self.seconds_to_empty())).strftime('%d.%B %H:%M')
        return '00:00'

    @get_item
    def seconds_since_full(self) -> int:
        """Seconds passed since full charge
            Returns:
                seconds as integer
        """
        return self._latest_details_data[self.IC_STATUS][self.SECONDS_SINCE_FULL_KEY]

    @get_item
    def installed_modules(self) -> int:
        """Battery modules installed in the system
            Returns:
                Number of modules
        """
        return self._ic_status[self.MODULES_INSTALLED_KEY]

    @get_item
    def system_status(self) -> str:
        return self._ic_status[self.SYSTEM_STATUS]

    @get_item
    def u_soc(self) -> int:
        """User state of charge
            Returns:
                User SoC in percent
        """
        return self._latest_details_data[self.USOC_KEY]

    @get_item
    def remaining_capacity_wh(self) -> int:
        """ Remaining capacity in watt hours
            IMPORTANT NOTE: Why is this double as high as it should be???
            Returns:
                 Remaining USABLE capacity of the battery in Wh
        """
        return self._status_data[self.REM_CON_WH_KEY] - 22000

    @get_item
    def full_charge_capacity(self) -> int:
        """Full charge capacity of the battery system
            Returns:
                Capacity in Wh
        """
        return self._latest_details_data[self.FULL_CHARGE_CAPACITY_KEY]

    def time_since_full(self) -> datetime.timedelta:
        """Calculates time since full charge.
           Returns:
               Time in format days hours minutes seconds
        """
        return datetime.timedelta(seconds=self.seconds_since_full())

    @get_item
    def seconds_remaining_to_fully_charged(self) -> int:
        """Time remaining until fully charged
            Returns:
                Time in seconds
        """
        remaining_charge = self.full_charge_capacity() - self.remaining_capacity_wh()
        if self.charging():
            return int(remaining_charge / self.charging()) * 3600
        return 0

    def fully_charged_at(self) -> datetime:
        """ Calculating time until fully charged """
        if self.charging():
            final_time = (datetime.datetime.now() + datetime.timedelta(seconds=self.seconds_remaining_to_fully_charged()))
            return final_time.strftime('%d.%B.%Y %H:%M')
        return 0

    @property
    def pac_total(self) -> int:
        """ Battery inverter load
            Negative if charging
            Positive if discharging
            Returns:
                  Inverter load value in watt
        """
        pac = self._latest_details_data.get(self.PAC_KEY)
        return int(pac) if pac else 0

    @get_item
    def charging(self) -> int:
        """Actual battery charging value
            Returns:
                Charging value in watt
        """
        if self.pac_total < 0:
            return abs(self.pac_total)
        return 0

    @get_item
    def discharging(self) -> int:
        """Actual battery discharging value
            Returns:
                Discharging value in watt
        """
        if self.pac_total > 0:
            return self.pac_total
        return 0

    @get_item
    def grid_in(self) -> int:
        """Actual grid feed in value
            Returns:
                Value in watt
        """
        if self._status_data[self.GRID_FEED_IN_WATT_KEY] > 0:
            return self._status_data[self.GRID_FEED_IN_WATT_KEY]
        return 0

    @get_item
    def grid_out(self) -> int:
        """Actual grid out value
            Returns:
                Value in watt
        """

        if self._status_data[self.GRID_FEED_IN_WATT_KEY] < 0:
            return abs(self._status_data[self.GRID_FEED_IN_WATT_KEY])
        return 0

    @get_item
    def battery_cycle_count(self) -> int:
        """Number of charge/discharge cycles
            Returns:
                Number of charge/discharge cycles
        """
        return self._battery_status[self.BATTERY_CYCLE_COUNT]

    @get_item
    def battery_full_charge_capacity(self) -> float:
        """Fullcharge capacity
            Returns:
                Fullcharge capacity in Ah
        """
        return self._battery_status[self.BATTERY_FULL_CHARGE_CAPACITY]

    @get_item
    def battery_max_cell_temp(self) -> float:
        """Max cell temperature
            Returns:
                Maximum cell temperature in ºC
        """
        return self._battery_status[self.BATTERY_MAX_CELL_TEMP]

    @get_item
    def battery_max_cell_voltage(self) -> float:
        """Max cell voltage
            Returns:
                Maximum cell voltage in Volt
        """
        return self._battery_status[self.BATTERY_MAX_CELL_VOLTAGE]

    @get_item
    def battery_max_module_current(self) -> float:
        """Max module DC current
            Returns:
                Maximum module DC current in Ampere
        """
        return self._battery_status[self.BATTERY_MAX_MODULE_CURRENT]

    @get_item
    def battery_max_module_voltage(self) -> float:
        """Max module DC voltage
            Returns:
                Maximum module DC voltage in Volt
        """
        return self._battery_status[self.BATTERY_MAX_MODULE_VOLTAGE]

    @get_item
    def battery_max_module_temp(self) -> float:
        """Max module DC temperature
            Returns:
                Maximum module DC temperature in ºC
        """
        return self._battery_status[self.BATTERY_MAX_MODULE_TEMP]

    @get_item
    def battery_min_cell_temp(self) -> float:
        """Min cell temperature
            Returns:
                Minimum cell temperature in ºC
        """
        return self._battery_status[self.BATTERY_MIN_CELL_TEMP]

    @get_item
    def battery_min_cell_voltage(self) -> float:
        """Min cell voltage
            Returns:
                Minimum cell voltage in Volt
        """
        return self._battery_status[self.BATTERY_MIN_CELL_VOLTAGE]

    @get_item
    def battery_min_module_current(self) -> float:
        """Min module DC current
            Returns:
                Minimum module DC current in Ampere
        """
        return self._battery_status[self.BATTERY_MIN_MODULE_CURRENT]

    @get_item
    def battery_min_module_voltage(self) -> float:
        """Min module DC voltage
            Returns:
                Minimum module DC voltage in Volt
        """
        return self._battery_status[self.BATTERY_MIN_MODULE_VOLTAGE]

    @get_item
    def battery_min_module_temp(self) -> float:
        """Min module DC temperature
            Returns:
                Minimum module DC temperature in ºC
        """
        return self._battery_status[self.BATTERY_MIN_MODULE_TEMP]

    @get_item
    def battery_rsoc(self) -> float:
        """Relative state of charge
            Returns:
                Relative state of charge in %
        """
        return self._battery_status[self.BATTERY_RSOC]

    @get_item
    def battery_remaining_capacity(self) -> float:
        """Remaining capacity
            Returns:
                Remaining capacity in Ah
        """
        return self._battery_status[self.BATTERY_REMAINING_CAPACITY]

    @get_item
    def battery_system_current(self) -> float:
        """System current
            Returns:
                System current in Ampere
        """
        return self._battery_status[self.BATTERY_SYSTEM_CURRENT]

    @get_item
    def battery_system_dc_voltage(self) -> float:
        """System battery voltage
            Returns:
                Voltage in Volt
        """
        return self._battery_status[self.BATTERY_SYSTEM_VOLTAGE]



import datetime
import os
import unittest
import responses
import logging
from freezegun import freeze_time
from sonnen_api_v2 import Sonnen


class TestSonnen(unittest.TestCase):

    @responses.activate
    def setUp(self) -> None:
        test_data_latest_charging = {
            'Consumption_W': 403,
            'FullChargeCapacity': 19531,
            'GridFeedIn_W': 0,
            'Pac_total_W': -94,
            'Production_W': 578,
            'RSOC': 98,
            'SetPoint_W': -145,
            'Timestamp': '2022-04-30 17:00:58',
            'USOC': 98,
            'UTC_Offet': 2,
            'ic_status': {
                'DC Shutdown Reason': {
                    'Critical BMS Alarm': False,
                    'Electrolyte Leakage': False,
                    'Error condition in BMS initialization': False,
                    'HW_Shutdown': False,
                    'HardWire Over Voltage': False,
                    'HardWired Dry Signal A': False,
                    'HardWired Under Voltage': False,
                    'Holding Circuit Error': False,
                    'Initialization Timeout': False,
                    'Initialization of AC contactor failed': False,
                    'Initialization of BMS hardware failed': False,
                    'Initialization of DC contactor failed': False,
                    'Initialization of Inverter failed': False,
                    'Invalid or no SystemType was set': False,
                    'Inverter Over Temperature': False,
                    'Inverter Under Voltage': False,
                    'Inverter Version Too Low For Dc-Module': False,
                    'Manual shutdown by user': False,
                    'Minimum rSOC of System reached': False,
                    'Modules voltage out of range': False,
                    'No Setpoint received by HC': False,
                    'Odd number of battery modules': False,
                    'One single module detected and module voltage is out of range': False,
                    'Only one single module detected': False,
                    'Shutdown Timer started': False,
                    'System Validation failed': False,
                    'Voltage Monitor Changed': False
                },
                'Eclipse Led': {
                    'Blinking Red': False,
                    'Pulsing Green': False,
                    'Pulsing Orange': False,
                    'Pulsing White': True,
                    'Solid Red': False
                },
                'MISC Status Bits': {
                    'Discharge not allowed': False,
                    'F1 open': False,
                    'Min System SOC': False,
                    'Min User SOC': False,
                    'Setpoint Timeout': False
                },
                'Microgrid Status': {
                    'Continious Power Violation': False,
                    'Discharge Current Limit Violation': False,
                    'Low Temperature': False,
                    'Max System SOC': False,
                    'Max User SOC': False,
                    'Microgrid Enabled': False,
                    'Min System SOC': False,
                    'Min User SOC': False,
                    'Over Charge Current': False,
                    'Over Discharge Current': False,
                    'Peak Power Violation': False,
                    'Protect is activated': False,
                    'Transition to Ongrid Pending': False
                },
                'Setpoint Priority': {
                    'BMS': False,
                    'Energy Manager': True,
                    'Full Charge Request': False,
                    'Inverter': False,
                    'Min User SOC': False,
                    'Trickle Charge': False
                },
                'System Validation': {
                    'Country Code Set status flag 1': False,
                    'Country Code Set status flag 2': False,
                    'Self test Error DC Wiring': False,
                    'Self test Postponed': False,
                    'Self test Precondition not met': False,
                    'Self test Running': False,
                    'Self test successful finished': False
                },
                'nrbatterymodules': 4,
                'secondssincefullcharge': 3720,
                'statebms': 'ready',
                'statecorecontrolmodule': 'ongrid',
                'stateinverter': 'running',
                'timestamp': 'Sat Apr 30 17:00:57 2022'
            }
        }

        test_data_status_discharging = {
            'Apparent_output': 438,
            'BackupBuffer': '0',
            'BatteryCharging': False,
            'BatteryDischarging': True,
            'Consumption_Avg': 563,
            'Consumption_W': 541,
            'Fac': 50.0167121887207,
            'FlowConsumptionBattery': True,
            'FlowConsumptionGrid': False,
            'FlowConsumptionProduction': True,
            'FlowGridBattery': False,
            'FlowProductionBattery': False,
            'FlowProductionGrid': False,
            'GridFeedIn_W': -20,
            'IsSystemInstalled': 1,
            'OperatingMode': '2',
            'Pac_total_W': 438,
            'Production_W': 102,
            'RSOC': 99,
            'RemainingCapacity_Wh': 41027,
            'Sac1': 438,
            'Sac2': None,
            'Sac3': None,
            'SystemStatus': 'OnGrid',
            'Timestamp': '2022-05-06 20:24:39',
            'USOC': 99,
            'Uac': 237,
            'Ubat': 211,
            'dischargeNotAllowed': False,
            'generator_autostart': False
        }

        test_data_latest_discharging = {
            'Consumption_W': 541,
            'FullChargeCapacity': 19531,
            'GridFeedIn_W': 0,
            'Pac_total_W': 439,
            'Production_W': 102,
            'RSOC': 99,
            'SetPoint_W': 439,
            'Timestamp': '2022-05-06 20:24:38',
            'USOC': 99,
            'UTC_Offet': 2,
            'ic_status': {
                'DC Shutdown Reason': {
                    'Critical BMS Alarm': False,
                    'Electrolyte Leakage': False,
                    'Error condition in BMS initialization': False,
                    'HW_Shutdown': False,
                    'HardWire Over Voltage': False,
                    'HardWired Dry Signal A': False,
                    'HardWired Under Voltage': False,
                    'Holding Circuit Error': False,
                    'Initialization Timeout': False,
                    'Initialization of AC contactor failed': False,
                    'Initialization of BMS hardware failed': False,
                    'Initialization of DC contactor failed': False,
                    'Initialization of Inverter failed': False,
                    'Invalid or no SystemType was set': False,
                    'Inverter Over Temperature': False,
                    'Inverter Under Voltage': False,
                    'Inverter Version Too Low For Dc-Module': False,
                    'Manual shutdown by user': False,
                    'Minimum rSOC of System reached': False,
                    'Modules voltage out of range': False,
                    'No Setpoint received by HC': False,
                    'Odd number of battery modules': False,
                    'One single module detected and module voltage is out of range': False,
                    'Only one single module detected': False,
                    'Shutdown Timer started': False,
                    'System Validation failed': False,
                    'Voltage Monitor Changed': False
                },
                'Eclipse Led': {
                    'Blinking Red': False,
                    'Pulsing Green': False,
                    'Pulsing Orange': False,
                    'Pulsing White': True,
                    'Solid Red': False
                },
                'MISC Status Bits': {
                    'Discharge not allowed': False,
                    'F1 open': False,
                    'Min System SOC': False,
                    'Min User SOC': False,
                    'Setpoint Timeout': False
                },
                'Microgrid Status': {
                    'Continious Power Violation': False,
                    'Discharge Current Limit Violation': False,
                    'Low Temperature': False,
                    'Max System SOC': False,
                    'Max User SOC': False,
                    'Microgrid Enabled': False,
                    'Min System SOC': False,
                    'Min User SOC': False,
                    'Over Charge Current': False,
                    'Over Discharge Current': False,
                    'Peak Power Violation': False,
                    'Protect is activated': False,
                    'Transition to Ongrid Pending': False
                },
                'Setpoint Priority': {
                    'BMS': False,
                    'Energy Manager': True,
                    'Full Charge Request': False,
                    'Inverter': False,
                    'Min User SOC': False,
                    'Trickle Charge': False
                },
                'System Validation': {
                    'Country Code Set status flag 1': False,
                    'Country Code Set status flag 2': False,
                    'Self test Error DC Wiring': False,
                    'Self test Postponed': False,
                    'Self test Precondition not met': False,
                    'Self test Running': False,
                    'Self test successful finished': False
                },
                'nrbatterymodules': 4,
                'secondssincefullcharge': 574,
                'statebms': 'ready',
                'statecorecontrolmodule': 'ongrid',
                'stateinverter': 'running',
                'timestamp': 'Fri May 6 20:24:36 2022'
            }
        }

        test_data_status_charging = {
            'Apparent_output': 98,
            'BackupBuffer': '0',
            'BatteryCharging': True,
            'BatteryDischarging': False,
            'Consumption_Avg': 486,
            'Consumption_W': 403,
            'Fac': 50.05781555175781,
            'FlowConsumptionBattery': False,
            'FlowConsumptionGrid': False,
            'FlowConsumptionProduction': True,
            'FlowGridBattery': False,
            'FlowProductionBattery': True,
            'FlowProductionGrid': True,
            'GridFeedIn_W': 54,
            'IsSystemInstalled': 1,
            'OperatingMode': '2',
            'Pac_total_W': -95,
            'Production_W': 578,
            'RSOC': 98,
            'RemainingCapacity_Wh': 40781,
            'Sac1': 98,
            'Sac2': None,
            'Sac3': None,
            'SystemStatus': 'OnGrid',
            'Timestamp': '2022-04-30 17:00:58',
            'USOC': 98,
            'Uac': 245,
            'Ubat': 212,
            'dischargeNotAllowed': False,
            'generator_autostart': False
        }

        test_data_powermeter = [
            {
                'a_l1': 2.4730000495910645, 'a_l2': 0,
                'a_l3': 0,
                'channel': 1,
                'deviceid': 4,
                'direction': 'production',
                'error': 0,
                'kwh_exported': 0,
                'kwh_imported': 3969.800048828125,
                'v_l1_l2': 0,
                'v_l1_n': 246.60000610351562,
                'v_l2_l3': 0,
                'v_l2_n': 0,
                'v_l3_l1': 0,
                'v_l3_n': 0,
                'va_total': 609.5,
                'var_total': 0,
                'w_l1': 609.5,
                'w_l2': 0,
                'w_l3': 0,
                'w_total': 609.5
            },
            {
                'a_l1': 2.0929999351501465,
                'a_l2': 0,
                'a_l3': 0,
                'channel': 2,
                'deviceid': 4,
                'direction': 'consumption',
                'error': 0,
                'kwh_exported': 0,
                'kwh_imported': 816.5,
                'v_l1_l2': 0,
                'v_l1_n': 246.6999969482422,
                'v_l2_l3': 0,
                'v_l2_n': 0,
                'v_l3_l1': 0,
                'v_l3_n': 0,
                'va_total': 516.2000122070312,
                'var_total': -512.7999877929688,
                'w_l1': 59.29999923706055,
                'w_l2': 0,
                'w_l3': 0,
                'w_total': 59.29999923706055
            }
        ]

        battery1_powermeter_data = responses.Response(
            method='GET',
            url='http://192.168.188.11/api/v2/powermeter',
            status=200,
            json=test_data_powermeter
        )

        battery1_latest_data = responses.Response(
            method='GET',
            url='http://192.168.188.11/api/v2/latestdata',
            status=200,
            json=test_data_latest_charging
        )

        battery1_status = responses.Response(
            method='GET',
            url='http://192.168.188.11/api/v2/status',
            status=200,
            json=test_data_status_charging
        )

        battery2_powermeter_data = responses.Response(
            method='GET',
            url='http://192.168.188.12/api/v2/powermeter',
            status=200,
            json=test_data_powermeter
        )

        battery2_latest_data = responses.Response(
            method='GET',
            url='http://192.168.188.12/api/v2/latestdata',
            status=200,
            json=test_data_latest_discharging
        )

        battery2_status = responses.Response(
            method='GET',
            url='http://192.168.188.12/api/v2/status',
            status=200,
            json=test_data_status_discharging
        )

        battery3_powermeter_data = responses.Response(
            method='GET',
            url='http://155.547.4.57/api/v2/powermeter',
            status=401,
            json={}
        )

        battery3_latest_data = responses.Response(
            method='GET',
            url='http://155.547.4.57/api/v2/latestdata',
            status=401,
            json={}
        )

        battery3_status = responses.Response(
            method='GET',
            url='http://155.547.4.57/api/v2/status',
            status=200,
            json=test_data_status_charging
        )

        responses.add(battery1_latest_data)
        responses.add(battery1_status)
        responses.add(battery1_powermeter_data)

        responses.add(battery2_latest_data)
        responses.add(battery2_status)
        responses.add(battery2_powermeter_data)

        responses.add(battery3_latest_data)
        responses.add(battery3_status)
        responses.add(battery3_powermeter_data)

        token = os.getenv('AUTH_TOKEN')
        self.battery_charging_working = Sonnen(token, '192.168.188.11')  # Working and charging
        self.battery_discharging_working = Sonnen(token, '192.168.188.12')  # Working and discharging
        self.battery_unreachable = Sonnen('xyZio', '155.156.19.5')  # Not Reachable
        self.battery_wrong_token_charging = Sonnen('notWorkingToken', '155.547.4.57')  # Wrong Token

        self.battery_charging_working.update()
        self.battery_unreachable.update()
        self.battery_wrong_token_charging.update()
        self.battery_discharging_working.update()

    @responses.activate
    def test_consumption_average(self):

        result1 = self.battery_charging_working.consumption_average()
        result2 = self.battery_unreachable.consumption_average()
        result3 = self.battery_wrong_token_charging.consumption_average()
        self.assertEqual(result1, 486)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 486)

    @responses.activate
    def test_consumption(self):
        self.battery_charging_working.update()
        self.battery_unreachable.update()
        self.battery_wrong_token_charging.update()
        result1 = self.battery_charging_working.consumption()
        result2 = self.battery_unreachable.consumption()
        result3 = self.battery_wrong_token_charging.consumption()
        self.assertEqual(result1, 403)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)

    @responses.activate
    def test_installed_modules(self):
        self.battery_charging_working.update()
        self.battery_unreachable.update()
        self.battery_wrong_token_charging.update()
        result1 = self.battery_charging_working.installed_modules()
        result2 = self.battery_unreachable.installed_modules()
        result3 = self.battery_wrong_token_charging.installed_modules()
        self.assertEqual(result1, 4)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)

    @responses.activate
    def test_discharging(self):
        result1 = self.battery_charging_working.discharging()
        result2 = self.battery_unreachable.discharging()
        result3 = self.battery_wrong_token_charging.discharging()
        result4 = self.battery_discharging_working.discharging()
        self.assertEqual(result1, 0)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 439)
        result1_pac = self.battery_charging_working.pac_total
        result2_pac = self.battery_unreachable.pac_total
        result3_pac = self.battery_wrong_token_charging.pac_total
        result4_pac = self.battery_discharging_working.pac_total
        self.assertLessEqual(result1_pac, 0)
        self.assertEqual(result2_pac, 0)
        self.assertLessEqual(result3_pac, 0)
        self.assertGreaterEqual(result4_pac, 0)

    @responses.activate
    def test_charging(self):
        result1 = self.battery_charging_working.charging()
        result2 = self.battery_unreachable.charging()
        result3 = self.battery_wrong_token_charging.charging()
        result4 = self.battery_discharging_working.charging()
        self.assertEqual(result1, 94)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 0)

    @responses.activate
    def test_grid_in(self):
        result1 = self.battery_charging_working.grid_in()
        result2 = self.battery_unreachable.grid_in()
        result3 = self.battery_wrong_token_charging.grid_in()
        result4 = self.battery_discharging_working.grid_in()
        self.assertEqual(result1, 54)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 54)
        self.assertEqual(result4, 0)

    @responses.activate
    def test_grid_out(self):
        result1 = self.battery_charging_working.grid_out()
        result2 = self.battery_unreachable.grid_out()
        result3 = self.battery_wrong_token_charging.grid_out()
        result4 = self.battery_discharging_working.grid_out()
        self.assertGreaterEqual(result1, 0)
        self.assertGreaterEqual(result2, 0)
        self.assertGreaterEqual(result3, 0)
        self.assertEqual(result4, 20)

    @responses.activate
    def test_production(self):
        result1 = self.battery_charging_working.production()
        result2 = self.battery_unreachable.production()
        result3 = self.battery_wrong_token_charging.production()
        result4 = self.battery_discharging_working.production()
        self.assertEqual(result1, 578)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 102)

    @responses.activate
    def test_usoc(self):
        result1 = self.battery_charging_working.u_soc()
        result2 = self.battery_unreachable.u_soc()
        result3 = self.battery_wrong_token_charging.u_soc()
        result4 = self.battery_discharging_working.u_soc()
        self.assertEqual(result1, 98)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 99)

    @responses.activate
    def test_seconds_to_empty(self):
        result1 = self.battery_charging_working.seconds_to_empty()
        result2 = self.battery_unreachable.seconds_to_empty()
        result3 = self.battery_wrong_token_charging.seconds_to_empty()
        result4 = self.battery_discharging_working.seconds_to_empty()
        self.assertEqual(result1, 0)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 156030)

    @responses.activate
    @freeze_time("24-05-2022 15:38:23")
    def test_fully_discharged_at(self):
        result1 = self.battery_charging_working.fully_discharged_at()
        result2 = self.battery_unreachable.fully_discharged_at()
        result3 = self.battery_wrong_token_charging.fully_discharged_at()
        result4 = self.battery_discharging_working.fully_discharged_at()
        self.assertEqual(result1, '00:00')
        self.assertEqual(result2, '00:00')
        self.assertEqual(result3, '00:00')
        self.assertEqual(result4, '26.May 10:58')

    @responses.activate
    @freeze_time("24-04-2022 15:38:23")
    def test_seconds_since_full(self):
        result1 = self.battery_charging_working.seconds_since_full()
        result2 = self.battery_unreachable.seconds_since_full()
        result3 = self.battery_wrong_token_charging.seconds_since_full()
        result4 = self.battery_charging_working.seconds_since_full()
        self.assertEqual(result1, 3720)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 3720)

    @responses.activate
    def test_full_charge_capacity(self):
        result1 = self.battery_charging_working.full_charge_capacity()
        result2 = self.battery_unreachable.full_charge_capacity()
        result3 = self.battery_wrong_token_charging.full_charge_capacity()
        result4 = self.battery_discharging_working.full_charge_capacity()
        self.assertEqual(result1, 19531)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 19531)

    @responses.activate
    @freeze_time('24-04-2022 15:38:23')
    def test_time_since_full(self):
        result1 = self.battery_charging_working.time_since_full()
        result2 = self.battery_unreachable.time_since_full()
        result3 = self.battery_wrong_token_charging.time_since_full()
        result4 = self.battery_charging_working.time_since_full()
        self.assertEqual(result1, datetime.timedelta(seconds=3720))
        self.assertEqual(result2, datetime.timedelta(seconds=0))
        self.assertEqual(result3, datetime.timedelta(seconds=0))
        self.assertEqual(result4, datetime.timedelta(seconds=3720))

    @responses.activate
    @freeze_time('24-04-2022 15:38:23')
    def test_time_remaining_to_fully_charged(self):
        result1 = self.battery_charging_working.seconds_remaining_to_fully_charged()
        result2 = self.battery_unreachable.seconds_remaining_to_fully_charged()
        result3 = self.battery_wrong_token_charging.seconds_remaining_to_fully_charged()
        result4 = self.battery_discharging_working.seconds_remaining_to_fully_charged()
        self.assertEqual(result1, 25200)
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 0)

    @responses.activate
    @freeze_time('24-04-2022 15:38:23')
    def test_fully_charged_at(self):
        result1 = self.battery_charging_working.fully_charged_at()
        result2 = self.battery_unreachable.fully_charged_at()
        result3 = self.battery_wrong_token_charging.fully_charged_at()
        result4 = self.battery_discharging_working.fully_charged_at()
        self.assertEqual(result1, '24.April.2022 22:38')
        self.assertEqual(result2, 0)
        self.assertEqual(result3, 0)
        self.assertEqual(result4, 0)



from setuptools import setup, find_packages
import os


def read_file(file):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, file), mode='r', encoding='UTF-8') as file:
        return file.read()


def get_version(file):
    for line in read_file(file).splitlines():
        if line.startswith('__version__'):
            delimiter = '"' if '"' in line else "'"
            return line.split(delimiter)[1]
    else:
        raise RuntimeError('Version not found!')


setup(
    name='sonnen-api-v2',
    version=get_version('sonnen_api_v2/__init__.py'),
    packages=find_packages(exclude='tests'),
    url='https://github.com/Katamave/sonnen_api_v2.git',
    license=read_file('LICENSE'),
    author='Vaclav Silhan',
    author_email='katamave@gmail.com',
    description=read_file('README.md'),
    install_requires=[
        'aiohttp>=3.8.4',
        'requests>=2.27.1'
    ]
)
